unit Main;

{

Coding a Game of Memory in Delphi  The User Interface

http://zarko-gajic.iz.hr/coding-a-game-of-memory-in-delphi-the-user-interface/

}

interface

uses
  MemoryGame,
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls, Vcl.StdCtrls,
  Vcl.Buttons;

type
  TMainForm = class(TForm)
    pnlGame: TPanel;
    pnlGameField: TPanel;
    pnlNewGame: TPanel;
    btnNewGame: TButton;
    pnlCurrentGame: TPanel;
    ledPairs: TLabeledEdit;
    ledPlayers: TLabeledEdit;
    gameGrid: TGridPanel;
    lblCurrentPlayerName: TLabel;
    txtCurrentPlayer: TStaticText;
    lblCurrentPlayerPairs: TLabel;
    txtStatistics: TStaticText;
    lbPlayerStatistics: TListBox;
    procedure btnNewGameClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    fMGame : TMemoryGame;
    function GetMGame: TMemoryGame;
    property MGame : TMemoryGame read GetMGame;

    procedure FieldClaimed(Sender : TObject; const mField : TMField);
    procedure OpenField(Sender : TObject; const mField : TMField);
    procedure CloseField(Sender : TObject; const mField : TMField);
    procedure FieldsPaired(Sender : TObject; const mField1, mField2 : TMField);
    procedure NextPlayer(Sender : TObject; const player : TPlayer);
    procedure GameOver(Sender : TObject; const player : TPlayer);
    procedure GameCreated(Sender : TObject);

    procedure UpdatePlayerStatistics;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

{ TMainForm }

procedure TMainForm.btnNewGameClick(Sender: TObject);
var
  newGamePairs, newGamePlayers : integer;
begin
  newGamePairs := StrToInt(ledPairs.Text);
  newGamePlayers := StrToInt(ledPlayers.Text);;

  MGame.NewGame(newGamePairs, newGamePlayers);
end;

procedure TMainForm.CloseField(Sender: TObject; const mField: TMField);
begin
  TButton(mField.Host).Caption := '?';
end;

procedure TMainForm.FieldClaimed(Sender: TObject; const mField: TMField);
begin
  TButton(mField.Host).AlignWithMargins := false;
  TButton(mField.Host).Refresh;
  Sleep(200);
  TButton(mField.Host).AlignWithMargins := true;
  TButton(mField.Host).Refresh;
end;


procedure TMainForm.FieldsPaired(Sender: TObject; const mField1, mField2: TMField);
begin
  TButton(mField1.Host).Font.Size := 8;
  TButton(mField1.Host).Caption := Format('- %s -', [mField1.Player.Name]);
  TButton(mField2.Host).Font.Size := 8;
  TButton(mField2.Host).Caption := Format('- %s -', [mField2.Player.Name]);

  TButton(mField1.Host).Margins.SetBounds(8,8,8,8);
  TButton(mField2.Host).Margins.SetBounds(8,8,8,8);

  TButton(mField1.Host).Font.Style := [];
  TButton(mField2.Host).Font.Style := [];

  lblCurrentPlayerPairs.Caption := 'Pairs: ' + mGame.CurrentPlayer.ClaimedPairs.ToString;

  UpdatePlayerStatistics;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  gameGrid.Caption := '';

  btnNewGame.Click;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  fMGame.Free;
end;

procedure TMainForm.GameCreated(Sender: TObject);
var
  i : integer;
  aButton : TButton;
begin
  UpdatePlayerStatistics;

  begin //prepare GridPanel UI
    gameGrid.RowCollection.BeginUpdate;
    gameGrid.ColumnCollection.BeginUpdate;

    for i := 0 to -1 + gameGrid.ControlCount do
      gameGrid.Controls[0].Free;

    gameGrid.RowCollection.Clear;
    gameGrid.ColumnCollection.Clear;

    for i := 1 to MGame.GameGridColumns do
      with gameGrid.RowCollection.Add do
      begin
        SizeStyle := ssPercent;
        Value := 100 / MGame.GameGridColumns;
      end;

    for i := 1 to MGame.GameGridRows do
      with gameGrid.ColumnCollection.Add do
      begin
        SizeStyle := ssPercent;
        Value := 100 / MGame.GameGridRows;
      end;

    //create playable hosts for fields
    for i := 0 to -1 + MGame.Fields.Count do
    begin
      aButton := TButton.Create(self);
      aButton.Parent := gameGrid;
      aButton.Visible := true;
      aButton.Font.Style := [fsBold];
      aButton.Font.Size := 20;
      aButton.Caption := '?'; //IntToStr(MGame.Fields[i].Value);
      aButton.Align := alClient;
      aButton.AlignWithMargins := true;

      MGame.Fields[i].Host := aButton;

      aButton.OnClick := MGame.FieldHostAction;
    end;

    gameGrid.RowCollection.EndUpdate;
    gameGrid.ColumnCollection.EndUpdate;
  end; //prepare GridPanel UI
end;

procedure TMainForm.GameOver(Sender: TObject; const player: TPlayer);
begin
  MessageDlg(Format('We have a winner: %s', [player.Name]), mtInformation, [mbOK], -1);
end;

function TMainForm.GetMGame: TMemoryGame;
begin
  if fMGame = nil then
  begin
    fMGame := TMemoryGame.Create;

    fMGame.OnOpenField := OpenField;
    fMGame.OnFieldClaimed := FieldClaimed;
    fMGame.OnFieldOpened := FieldClaimed;
    fMGame.OnCloseField := CloseField;
    fMGame.OnFieldsPaired := FieldsPaired;
    fMGame.OnNextPlayer := NextPlayer;
    fMGame.OnGameStart := NextPlayer;
    fMGame.OnGameOver := GameOver;
    fMGame.OnGameCreated := GameCreated;

  end;
  result := fMGame;
end;

procedure TMainForm.NextPlayer(Sender: TObject; const player: TPlayer);
begin
  txtCurrentPlayer.Font.Style := [fsBold, fsUnderline];
  txtCurrentPlayer.Refresh;
  Sleep(100);
  txtCurrentPlayer.Font.Style := [fsBold];
  txtCurrentPlayer.Refresh;


  lblCurrentPlayerName.Caption := player.Name;
  lblCurrentPlayerPairs.Caption := 'Pairs: ' + player.ClaimedPairs.ToString;

  Caption := Format('Delphi Memory 2D. Current player: %s', [player.Name]);
end;

procedure TMainForm.OpenField(Sender: TObject; const mField: TMField);
begin
  TButton(mField.Host).Caption := mField.Value.ToString();
end;

procedure TMainForm.UpdatePlayerStatistics;
var
  player : TPlayer;
begin
  lbPlayerStatistics.Items.BeginUpdate;
  try
    lbPlayerStatistics.Clear;

    for player in MGame.Players do
      lbPlayerStatistics.Items.Add(Format('%s: %d', [player.Name, player.ClaimedPairs]));
  finally
    lbPlayerStatistics.Items.EndUpdate;
  end;
end;

end.
